from __future__ import annotations

import json
import logging
from typing import Dict, List, Optional, Union

from django import forms
from django.contrib import messages
from django.core.validators import validate_email
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _lazy

from .models import AppmailMessage, EmailTemplate, EmailTemplateQuerySet

logger = logging.getLogger(__name__)


class JSONWidget(forms.Textarea):
    """Pretty print JSON in a text area."""

    DEFAULT_ATTRS = {
        "class": "vLargeTextField",
        "rows": 15,
    }  # type: Dict[str, Union[str, int]]

    def format_value(self, value: str) -> str:
        """Pretty format JSON text."""
        value = value or "{}"
        if not isinstance(value, str):
            raise TypeError("Value must JSON parseable string instance")
        value = json.loads(value)
        return json.dumps(value, indent=4, sort_keys=True)

    def render(
        self,
        name: str,
        value: str,
        attrs: Optional[Dict[str, Union[str, int]]] = None,
        renderer: Optional[forms.renderers.BaseRenderer] = None,
    ) -> str:
        attrs = attrs or JSONWidget.DEFAULT_ATTRS
        value = self.format_value(value)
        return super().render(name, value, attrs=attrs, renderer=renderer)


class MultiEmailField(forms.Field):
    """Taken from https://docs.djangoproject.com/en/1.11/ref/forms/validation/#form-field-default-cleaning """  # noqa

    def to_python(self, value: Optional[Union[List[str], str]]) -> List[str]:
        """Normalize data to a list of strings."""
        if isinstance(value, list):
            return value

        if not value:
            return []

        return [v.strip() for v in value.split(",")]

    def validate(self, value: List[str]) -> None:
        """Check if value consists only of valid emails."""
        # Use the parent's handling of required fields, etc.
        super().validate(value)
        for email in value:
            validate_email(email)


class MultiEmailTemplateField(forms.Field):
    """Convert comma-separated ids into EmailTemplate queryset."""

    def to_python(
        self, value: Optional[Union[EmailTemplateQuerySet, str]]
    ) -> EmailTemplateQuerySet:
        """Normalize data to a queryset of EmailTemplates."""
        if isinstance(value, EmailTemplateQuerySet):
            return value

        if not value:
            return EmailTemplate.objects.none()

        values = [int(i) for i in value.split(",")]
        return EmailTemplate.objects.filter(pk__in=values)


class EmailTestForm(forms.Form):
    """Renders email template on intermediate page."""

    from_email = forms.EmailField(
        label=_lazy("From"), help_text=_lazy("Email address to be used as the sender")
    )
    reply_to = MultiEmailField(
        label=_lazy("Reply-To"),
        help_text=_lazy("Comma separated list of email addresses"),
    )
    to = MultiEmailField(
        label=_lazy("To"), help_text=_lazy("Comma separated list of email addresses")
    )
    cc = MultiEmailField(
        label=_lazy("cc"),
        help_text=_lazy("Comma separated list of email addresses"),
        required=False,
    )
    bcc = MultiEmailField(
        label=_lazy("bcc"),
        help_text=_lazy("Comma separated list of email addresses"),
        required=False,
    )
    context = forms.CharField(
        widget=forms.Textarea,
        required=False,
        help_text=_lazy("JSON used to render the subject and body templates"),
    )
    # comma separated list of template ids.
    templates = MultiEmailTemplateField(widget=forms.HiddenInput())

    def clean_context(self) -> dict:
        """Load text input back into JSON."""
        context = self.cleaned_data["context"] or "{}"
        try:
            return json.loads(context)
        except (TypeError, ValueError) as ex:
            raise forms.ValidationError(_lazy("Invalid JSON: %s" % ex))

    def _create_message(self, template: EmailTemplate) -> AppmailMessage:
        """Create EmailMultiMessage from form data."""
        return AppmailMessage(
            template,
            self.cleaned_data["context"],
            from_email=self.cleaned_data["from_email"],
            to=self.cleaned_data["to"],
            cc=self.cleaned_data["cc"],
            bcc=self.cleaned_data["bcc"],
        )

    def send_emails(self, request: HttpRequest) -> None:
        """Send test emails."""
        for template in self.cleaned_data.get("templates"):
            email = self._create_message(template)
            try:
                email.send()
            except Exception as ex:  # pylint: disable=broad-except
                logger.exception("Error sending test email")
                messages.error(
                    request,
                    _lazy("Error sending test email '%s': %s" % (template.name, ex)),
                )
            else:
                messages.success(
                    request,
                    _lazy(
                        "'%s' email sent to '%s'" % (template.name, ", ".join(email.to))
                    ),
                )



# Generated by Django 2.2.7 on 2019-11-12 07:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("appmail", "0005_emailtemplate_from_email__reply_to")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="supports_attachments",
            field=models.BooleanField(
                default=False,
                help_text="Does this template support file attachments?",
                verbose_name="Supports attachments",
            ),
        )
    ]



from __future__ import annotations

from typing import Any, Callable, Dict, List, Optional

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.core.mail import EmailMultiAlternatives
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models, transaction
from django.http import HttpRequest
from django.template import Context, Template, TemplateDoesNotExist, TemplateSyntaxError
from django.utils.timezone import now as tz_now
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy as _lazy

from . import helpers
from .compat import JSONField
from .settings import (
    ADD_EXTRA_HEADERS,
    CONTEXT_PROCESSORS,
    LOG_SENT_EMAILS,
    VALIDATE_ON_SAVE,
)

User = get_user_model()


class EmailTemplateQuerySet(models.query.QuerySet):
    def active(self) -> EmailTemplateQuerySet:
        """Return active templates only."""
        return self.filter(is_active=True)

    def current(
        self, name: str, language: str = settings.LANGUAGE_CODE
    ) -> EmailTemplateQuerySet:
        """Return the latest version of a template."""
        return (
            self.active()
            .filter(name=name, language=language)
            .order_by("version")
            .last()
        )

    def version(
        self, name: str, version: str, language: str = settings.LANGUAGE_CODE
    ) -> EmailTemplate:
        """Return a specific version of a template."""
        return self.active().get(name=name, language=language, version=version)


class EmailTemplate(models.Model):
    """
    Email template. Contains HTML and plain text variants.

    Each Template object has a unique name:language.version combination, which
    means that localisation of templates is managed through having multiple
    objects with the same name - there is no inheritence model. This is to
    keep it simple:

        order-confirmation:en.0
        order-confirmation:de.0
        order-confirmation:fr.0

    Templates contain HTML and plain text content.

    """

    CONTENT_TYPE_PLAIN = "text/plain"
    CONTENT_TYPE_HTML = "text/html"
    CONTENT_TYPES = (CONTENT_TYPE_PLAIN, CONTENT_TYPE_HTML)

    name = models.CharField(
        _lazy("Template name"),
        max_length=100,
        help_text=_lazy(
            "Template name - must be unique for a given language/version combination."
        ),
        db_index=True,
    )
    description = models.CharField(
        _lazy("Description"),
        max_length=100,
        help_text=_lazy(
            "Optional description. e.g. used to differentiate variants ('new header')."
        ),  # noqa
        blank=True,
    )
    # language is free text and not a choices field as we make no assumption
    # as to how the end user is storing / managing languages.
    language = models.CharField(
        _lazy("Language"),
        max_length=20,
        default=settings.LANGUAGE_CODE,
        help_text=_lazy(
            "Used to support localisation of emails, defaults to "
            "`settings.LANGUAGE_CODE`, but can be any string, e.g. 'London', 'NYC'."
        ),
        db_index=True,
    )
    version = models.IntegerField(
        _lazy("Version (or variant)"),
        default=0,
        help_text=_lazy("Integer value - can be used for versioning or A/B testing."),
        db_index=True,
    )
    subject = models.CharField(
        _lazy("Subject line template"),
        max_length=100,
        help_text=_lazy("Email subject line (may contain template variables)."),
    )
    body_text = models.TextField(
        _lazy("Plain text template"),
        help_text=_lazy("Plain text content (may contain template variables)."),
    )
    body_html = models.TextField(
        _lazy("HTML template"),
        help_text=_lazy("HTML content (may contain template variables)."),
    )
    test_context = JSONField(
        default=dict,
        blank=True,
        help_text=_lazy(
            "Dummy JSON used for test rendering (set automatically on first save)."
        ),
    )
    is_active = models.BooleanField(
        _lazy("Active (live)"),
        help_text=_lazy("Set to False to remove from `current` queryset."),
        default=True,
    )
    from_email = models.CharField(
        _lazy("Sender"),
        max_length=254,
        help_text=_lazy(
            "Default sender address if none specified. Verbose form is accepted."
        ),
        default=settings.DEFAULT_FROM_EMAIL,
    )
    reply_to = models.CharField(
        _lazy("Reply-To"),
        max_length=254,
        help_text=_lazy("Comma separated list of Reply-To recipients."),
        default=settings.DEFAULT_FROM_EMAIL,
    )
    supports_attachments = models.BooleanField(
        _lazy("Supports attachments"),
        default=False,
        help_text=_lazy("Does this template support file attachments?"),
    )

    objects = EmailTemplateQuerySet().as_manager()

    class Meta:
        unique_together = ("name", "language", "version")

    def __str__(self) -> str:
        return f"{self.name} (language={self.language}; version={self.version})"

    def __repr__(self) -> str:
        return (
            f"<EmailTemplate id={self.id} name='{self.name}' "
            f"language='{self.language}' version={self.version}>"
        )

    @property
    def extra_headers(self) -> Dict[str, str]:
        return {
            "X-Appmail-Template": (
                f"name={self.name}; language={self.language}; version={self.version}"
            )
        }

    @property
    def reply_to_list(self) -> List[str]:
        """Convert the reply_to field to a list."""
        return [a.strip() for a in self.reply_to.split(",")]

    def save(self, *args: Any, **kwargs: Any) -> EmailTemplate:
        """
        Update dummy context on first save and validate template contents.

        Kwargs:
            validate: set to False to bypass template validation; defaults
                to settings.VALIDATE_ON_SAVE.

        """
        if self.pk is None:
            self.test_context = helpers.get_context(
                self.subject + self.body_text + self.body_html
            )
        validate = kwargs.pop("validate", VALIDATE_ON_SAVE)
        if validate:
            self.clean()
        super(EmailTemplate, self).save(*args, **kwargs)
        return self

    def clean(self) -> None:
        """Validate model - specifically that the template can be rendered."""
        validation_errors = {}
        validation_errors.update(self._validate_body(EmailTemplate.CONTENT_TYPE_PLAIN))
        validation_errors.update(self._validate_body(EmailTemplate.CONTENT_TYPE_HTML))
        validation_errors.update(self._validate_subject())
        if validation_errors:
            raise ValidationError(validation_errors)

    def render_subject(
        self,
        context: dict,
        processors: List[Callable[[HttpRequest], dict]] = CONTEXT_PROCESSORS,
    ) -> str:
        """Render subject line."""
        ctx = Context(helpers.patch_context(context, processors), autoescape=False)
        return Template(self.subject).render(ctx)

    def _validate_subject(self) -> Dict[str, str]:
        """Try rendering the body template and capture any errors."""
        try:
            self.render_subject({})
        except TemplateDoesNotExist as ex:
            return {"subject": _lazy("Template does not exist: {}".format(ex))}
        except TemplateSyntaxError as ex:
            return {"subject": str(ex)}
        else:
            return {}

    def render_body(
        self,
        context: dict,
        content_type: str = CONTENT_TYPE_PLAIN,
        processors: List[Callable[[HttpRequest], dict]] = CONTEXT_PROCESSORS,
    ) -> str:
        """Render email body in plain text or HTML format."""
        if content_type not in EmailTemplate.CONTENT_TYPES:
            raise ValueError(_(f"Invalid content type. Value supplied: {content_type}"))
        if content_type == EmailTemplate.CONTENT_TYPE_PLAIN:
            ctx = Context(helpers.patch_context(context, processors), autoescape=False)
            return Template(self.body_text).render(ctx)
        if content_type == EmailTemplate.CONTENT_TYPE_HTML:
            ctx = Context(helpers.patch_context(context, processors))
            return Template(self.body_html).render(ctx)
        raise ValueError(f"Invalid content_type '{content_type}'.")

    def _validate_body(self, content_type: str) -> Dict[str, str]:
        """Try rendering the body template and capture any errors."""
        if content_type == EmailTemplate.CONTENT_TYPE_PLAIN:
            field_name = "body_text"
        elif content_type == EmailTemplate.CONTENT_TYPE_HTML:
            field_name = "body_html"
        else:
            raise ValueError("Invalid template content_type.")
        try:
            self.render_body({}, content_type=content_type)
        except TemplateDoesNotExist as ex:
            return {field_name: _("Template does not exist: {}".format(ex))}
        except TemplateSyntaxError as ex:
            return {field_name: str(ex)}
        else:
            return {}

    def clone(self) -> EmailTemplate:
        """Create a copy of the current object, increase version by 1."""
        self.pk = None
        self.version += 1
        return self.save()


class AppmailMessage(EmailMultiAlternatives):
    """
    Subclass EmailMultiAlternatives to be template-aware.

    This class behaves like a standard Django EmailMultiAlternaives
    email, but with the subject, body and 'text/html' alternative set
    from the template + context.

    The send method is overridden to enable the saving a copy of the
    message after it is sent.

    """

    def __init__(self, template: EmailTemplate, context: dict, **email_kwargs: Any):
        """
        Build AppmailMessage from template and context.

        The template and context are used to render the email subject line,
        body and HTML.

        The `email_kwargs` are passed direct to the EmailMultiAlternatives
        constructor - so can be anything that that supports (cc, bcc, etc.)

        """
        if "subject" in email_kwargs:
            raise ValueError(_("Invalid argument: 'subject' is set from the template."))
        if "body" in email_kwargs:
            raise ValueError(_("Invalid argument: 'body' is set from the template."))
        if "alternatives" in email_kwargs:
            raise ValueError(
                _("Invalid argument: 'alternatives' is set from the template.")
            )

        email_kwargs.setdefault("reply_to", template.reply_to_list)
        email_kwargs.setdefault("from_email", template.from_email)
        email_kwargs.setdefault("headers", {})
        if ADD_EXTRA_HEADERS:
            email_kwargs["headers"].update(template.extra_headers)

        if email_kwargs.get("attachments", None) and not template.supports_attachments:
            raise ValueError(_("Email template does not support attachments."))

        email_kwargs["subject"] = template.render_subject(context)
        email_kwargs["body"] = template.render_body(
            context, content_type=EmailTemplate.CONTENT_TYPE_PLAIN
        )
        html = template.render_body(
            context, content_type=EmailTemplate.CONTENT_TYPE_HTML
        )
        email_kwargs["alternatives"] = [(html, EmailTemplate.CONTENT_TYPE_HTML)]

        super().__init__(**email_kwargs)

        self.template = template
        self.context = context
        self.html = html

    def size_in_bytes(self) -> int:
        """Return the size of the underlying message in bytes."""
        return len(self.message().as_bytes())

    def _user(self, email: str) -> Optional[settings.AUTH_USER_MODEL]:
        """
        Fetch a user matching the 'to' email address.

        This method assumes that emails are unique. If users share emails,
        then we return None, as we cannot determine who is the correct
        recipient.

        """
        email = email.strip().lower()
        if not email:
            return None
        try:
            return User.objects.get(email__iexact=email)
        except (User.DoesNotExist, User.MultipleObjectsReturned):
            return None

    @transaction.atomic
    def send(
        self,
        *,
        log_sent_emails: bool = LOG_SENT_EMAILS,
        fail_silently: bool = False,
    ) -> int:
        """
        Send the email and add to audit log.

        This method first sends the email using the underlying
        send method inherited from EmailMultiMessage. If any messages
        are sent (return value > 0), then the message is logged.

        """
        sent = super().send(fail_silently=fail_silently)
        if not log_sent_emails:
            return sent
        if not sent:
            return 0
        return len(LoggedMessage.objects.log(self))


class LoggedMessageManager(models.Manager):
    def log(self, message: AppmailMessage) -> List[LoggedMessage]:
        """Log the sending of emails from an AppmailMessage."""
        return [
            LoggedMessage.objects.create(
                template=message.template,
                to=email,
                user=message._user(email),
                subject=message.subject,
                body=message.body,
                html=message.html,
                context=message.context,
            )
            for email in message.to
        ]


class LoggedMessage(models.Model):
    """Record of emails sent via Appmail."""

    # ensure we record the email address itself, even if we don't have a User object
    to = models.EmailField(help_text=_lazy("Address to which the the Email was sent."))
    # nullable, as sometimes we may have unknown senders / recipients?
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="logged_emails",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        db_index=True,
    )
    template: EmailTemplate = models.ForeignKey(
        EmailTemplate,
        related_name="logged_emails",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        db_index=True,
        help_text=_lazy("The appmail template used."),
    )
    timestamp = models.DateTimeField(
        default=tz_now, help_text=_lazy("When the email was sent."), db_index=True
    )
    subject = models.TextField(blank=True, help_text=_lazy("Email subject line."))
    body = models.TextField(
        "Plain text", blank=True, help_text=_lazy("Plain text content.")
    )
    html = models.TextField("HTML", blank=True, help_text=_lazy("HTML content."))
    context = JSONField(
        default=dict,
        encoder=DjangoJSONEncoder,
        help_text=_lazy("Appmail template context."),
    )

    objects = LoggedMessageManager()

    class Meta:
        get_latest_by = "timestamp"
        verbose_name = "Email message"
        verbose_name_plural = "Email messages sent"

    def __repr__(self) -> str:
        return (
            f"<LoggedMessage id:{self.id} template='{self.template_name}' "
            f"to='{self.to}'>"
        )

    def __str__(self) -> str:
        return f"LoggedMessage sent to {self.to} ['{self.template_name}']>"

    @property
    def template_name(self) -> str:
        """Return the name of the template used."""
        if not self.template:
            return ""
        return self.template.name

    def rehydrate(self) -> AppmailMessage:
        """Create a new AppmailMessage message from this email."""
        return AppmailMessage(
            template=self.template, context=self.context, to=[self.to]
        )

    def resend(
        self,
        log_sent_emails: bool = LOG_SENT_EMAILS,
        fail_silently: bool = False,
    ) -> None:
        """Recreate new AppmailMessage from this log and send it."""
        self.rehydrate().send(
            log_sent_emails=log_sent_emails,
            fail_silently=fail_silently,
        )



# Generated by Django 3.1.5 on 2021-01-04 06:48

import django.core.serializers.json
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

from ..compat import JSONField


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("appmail", "0006_emailtemplate_supports_attachments"),
    ]

    operations = [
        migrations.CreateModel(
            name="LoggedMessage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "to",
                    models.EmailField(
                        help_text="Address to which the the Email was sent.",
                        max_length=254,
                    ),
                ),
                (
                    "timestamp",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="When the email was sent.",
                        db_index=True,
                    ),
                ),
                (
                    "subject",
                    models.TextField(blank=True, help_text="Email subject line."),
                ),
                (
                    "body",
                    models.TextField(
                        blank=True,
                        help_text="Plain text content.",
                        verbose_name="Plain text",
                    ),
                ),
                (
                    "html",
                    models.TextField(
                        blank=True, help_text="HTML content.", verbose_name="HTML"
                    ),
                ),
                (
                    "context",
                    JSONField(
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        help_text="Appmail template context.",
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        blank=True,
                        help_text="The appmail template used.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="logged_emails",
                        to="appmail.emailtemplate",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="logged_emails",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "get_latest_by": "timestamp",
                "verbose_name": "Email message",
                "verbose_name_plural": "Email messages sent",
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-09-11 03:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("appmail", "0004_emailtemplate_is_active")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="from_email",
            field=models.CharField(
                default=settings.DEFAULT_FROM_EMAIL,
                help_text=(
                    "Default sender address if none specified. Verbose form is "
                    "accepted."
                ),
                max_length=254,
                verbose_name="Sender",
            ),
        ),
        migrations.AddField(
            model_name="emailtemplate",
            name="reply_to",
            field=models.CharField(
                default=settings.DEFAULT_FROM_EMAIL,
                help_text="Comma separated list of Reply-To recipients.",
                max_length=254,
                verbose_name="Reply-To",
            ),
        ),
        migrations.AlterField(
            model_name="emailtemplate",
            name="description",
            field=models.CharField(
                blank=True,
                help_text=(
                    "Optional description. e.g. used to differentiate variants ('new "
                    "header')."
                ),
                max_length=100,
                verbose_name="Description",
            ),
        ),
    ]



from django.conf import settings
from django.utils.module_loading import import_string

# Validate that EmailTemplate can be rendered without
# error on each save. Defaults to True.
VALIDATE_ON_SAVE = getattr(settings, "APPMAIL_VALIDATE_ON_SAVE", True)
# if True then add X-Appmail-* headers to outgoung email objects
ADD_EXTRA_HEADERS = getattr(settings, "APPMAIL_ADD_HEADERS", True)
# list of context processor functions applied on each render
CONTEXT_PROCESSORS = [
    import_string(s) for s in getattr(settings, "APPMAIL_CONTEXT_PROCESSORS", [])
]  # noqa

# If True then emails will be logged.
LOG_SENT_EMAILS = getattr(settings, "APPMAIL_LOG_SENT_EMAILS", True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['appmail', 'appmail.migrations']

package_data = \
{'': ['*'],
 'appmail': ['templates/*',
             'templates/admin/appmail/emailtemplate/*',
             'templates/admin/appmail/loggedmessage/*',
             'templates/appmail/*',
             'tests/fixtures/*']}

install_requires = \
['django>=3.0,<4.0']

setup_kwargs = {
    'name': 'django-appmail',
    'version': '2.0b7',
    'description': 'Django app for managing localised email templates.',
    'long_description': '# Django-AppMail\n\nDjango app for managing transactional email templates.\n\n## Compatibility\n\nThis project now requires Django 3.0+ and Python 3.7+. If you require a previous\nversion you will have to refer to the relevant branch or tag.\n\n## Background\n\nThis project arose out of a project to integrate a large transactional Django\napplication with Mandrill, and the lessons learned. It also owes a minor h/t to\nthis project from 2011 (https://github.com/hugorodgerbrown/AppMail).\n\nThe core requirement is to provide an easy way to add / edit email templates to\na Django project, in such a way that it doesn\'t require a developer to make\nchanges. The easiest way to use templated emails in Django is to rely on the\nin-built template structure, but that means that the templates are held in\nfiles, under version control, which makes it very hard for non-developers to\nedit.\n\nThis is **not** a WYSIWYG HTML editor, and it doesn\'t do anything clever. It\ndoesn\'t handle the sending of the emails - it simply provides a convenient\nmechanism for storing and rendering email content.\n\n```python\nfrom appmail.models import EmailTemplate, AppmailMessage\n\ndef send_order_confirmation(order_id):\n    order = Orders.objects.get(id=order_id)\n    template = EmailTemplate.objects.current(\'order_confirmation\')\n    context = { "order": order }\n    message = AppmailMessage(\n        template=template,\n        context=context,\n        to=[order.recipient.email]\n    )\n    message.send()\n```\n\nThe core requirements are:\n\n1. List / preview existing templates\n2. Edit subject line, plain text and HTML content\n3. Use standard Django template syntax\n4. Support base templates\n5. Template versioning\n6. Language support\n7. Send test emails\n8. Log emails sent (if desired)\n\n### Email logging (v2)\n\nFrom v2 on, it is possible to log all emails that are sent via\n`AppmailMessage.send`. It records the template, context and the rendered output,\nso that the email can be views as sent, and resent. It will attempt to record\nthe User to whom the email was sent, as well as the email address. This is\ndependent on there being a unique 1:1 match from email to User object, but can\nprove useful in tracking emails sent to users when they change their email\naddress.\n\n### Template properties\n\nIndividual templates are stored as model objects in the database. The standard\nDjango admin site is used to view / filter templates. The templates are ordered\nby name, language and version. This combination is unique. The language and\nversion properties have sensible defaults (`version=settings.LANGUAGE_CODE` and\n`version=0`) so don\'t need to set if you don\'t require it. There is no\ninheritance or relationship between different languages and versions - they are\nstored as independent objects.\n\n```python\n# get the default order_summary email (language = settings.LANGUAGE_CODE)\ntemplate = EmailTemplate.objects.current(\'order_summary\')\n# get the french version\ntemplate = EmailTemplate.objects.current(\'order_summary\', language=\'fr\')\n# get a specific version\ntemplate = EmailTemplate.objects.version(\'order_summary\', 1)\n```\n\n**Template syntax**\n\nThe templates themselves use standard Django template syntax, including the use\nof tags, filters. There is nothing special about them, however there is one\ncaveat - template inheritance.\n\n**Template inheritance**\n\nAlthough the template content is not stored on disk, without re-engineering the\ntemplate rendering methods any parent templates must be. This is annoying, but\nthere is a valid assumption behind it - if you are changing your base templates\nyou are probably involving designers and developers already, so having to rely\non a developer to make the changes is acceptable.\n\n**Sending test emails**\n\nYou can send test emails to an email address through the admin list view.\n\n<img src="screenshots/appmail-test-email-action.png" alt="EmailTemplate admin\nchange form" />\n\nThe custom admin action \'Send test emails\' will redirect to an intermediate page\nwhere you can enter the recipient email address and send the email:\n\n<img src="screenshots/appmail-test-email-send.png"/>\n\nThere is also a linkon individual template admin pages (top-right, next to the\nhistory link):\n\n<img src="screenshots/appmail-template-change-form.png" alt="EmailTemplate admin\nchange form" />\n\n## Tests\n\nThere is a test suite for the app, which is best run through `tox`.\n\n## License\n\nMIT\n\n## Contributing\n\nUsual rules apply:\n\n1. Fork to your own account\n2. Fix the issue / add the feature\n3. Submit PR\n\nPlease take care to follow the coding style - and PEP8.\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-03-11 11:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="EmailTemplate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text=(
                            "Template name - must be unique for a given language/"
                            "version combination."
                        ),
                        max_length=100,
                        verbose_name="Template name",
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        db_index=True,
                        default="en-us",
                        help_text=(
                            "Used to support localisation of emails, defaults to "
                            "settings.LANGUAGE_CODE."
                        ),
                        max_length=20,
                        verbose_name="Language code",
                    ),
                ),
                (
                    "version",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text=(
                            "Integer value - can be used for versioning or A/B testing."
                        ),
                        verbose_name="Version (or variant)",
                    ),
                ),
                (
                    "subject",
                    models.CharField(
                        help_text=(
                            "Email subject line (may contain template variables)."
                        ),
                        max_length=100,
                        verbose_name="Subject line template",
                    ),
                ),
                (
                    "body_text",
                    models.TextField(
                        help_text=(
                            "Plain text content (may contain template variables)."
                        ),
                        verbose_name="Plain text template",
                    ),
                ),
                (
                    "body_html",
                    models.TextField(
                        help_text="HTML content (may contain template variables).",
                        verbose_name="HTML template",
                    ),
                ),
            ],
        ),
        migrations.AlterUniqueTogether(
            name="emailtemplate", unique_together=set([("name", "language", "version")])
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-15 03:26
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("appmail", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="description",
            field=models.CharField(
                blank=True,
                help_text=(
                    "Optional description. e.g. used to differentiate variants ('new "
                    "header')."
                ),
                max_length=100,
            ),
        ),
        migrations.AlterField(
            model_name="emailtemplate",
            name="language",
            field=models.CharField(
                db_index=True,
                default=settings.LANGUAGE_CODE,
                help_text=(
                    "Used to support localisation of emails, defaults to "
                    "`settings.LANGUAGE_CODE`, but can be any string, e.g. 'London', "
                    "'NYC'."
                ),
                max_length=20,
                verbose_name="Language",
            ),
        ),
    ]
